==> thesis-1.0-20150607.011054-1-sources.jar: com/whimsy/map/algo/AStar.java <==
package com.whimsy.map.algo;

import java.util.List;
import java.util.PriorityQueue;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.whimsy.map.base.Constant;
import com.whimsy.map.base.Edge;
import com.whimsy.map.base.Node;

import edu.princeton.cs.algs4.Bag;


import static com.whimsy.map.base.Utils.*;

/**
 * Created by whimsy on 6/1/15.
 */
public class AStar {
    static final Logger LOG = LoggerFactory.getLogger(AStar.class);

    Bag<Edge>[] bags;

    List<Node> nodes;

    @SuppressWarnings("unchecked")
    public AStar(List<Node> nodes, List<Edge> edges) {

        this.nodes = nodes;

        bags = new Bag[nodes.size()];

        for (int i = 0; i < nodes.size(); ++i) {
            bags[i] = new Bag<Edge>();
        }

        for (Edge edge : edges) {
            bags[edge.sId].add(edge);
        }
    }



    public double query(int s, int t) {

        PriorityQueue<HeapNode> pq = new PriorityQueue<HeapNode>();

        int n = bags.length;


        double [] d = new double[n];
        for (int i = 0; i < n; ++i) {
            d[i] = Constant.INF;
        }



        // initialize source node

        d[s] = 0;
        pq.add(new HeapNode(s, d[s] + estimateDist(s, t)));

        boolean [] isClosed = new boolean[n];

        while (!pq.isEmpty()) {
            HeapNode cur = pq.poll();

            if (isClosed[cur.idx]) continue;

            isClosed[cur.idx] = true;
            if (cur.idx == t) {
                break;
            }

            for (Edge edge : bags[cur.idx]) {
                if (d[edge.eId] > d[cur.idx] + edge.len()) {
                    d[edge.eId]  = d[cur.idx] + edge.len();
                    pq.add(new HeapNode(edge.eId, d[edge.eId] + estimateDist(edge.eId, t)));
                }
            }
        }

        LOG.debug("AStar distance : {}", d[t]);


        return d[t];

    }

    private double estimateDist(int s, int t) {
        return dist(nodes.get(s).lat, nodes.get(s).lon, nodes.get(t).lat, nodes.get(t).lon);

    }

    class HeapNode implements  Comparable<HeapNode> {

        int idx;
        double value;

        public HeapNode(int idx, double value) {
            this.idx = idx;
            this.value = value;
        }

        @Override
        public int compareTo(HeapNode o) {
            return value == o.value ? 0 : (value < o.value ? -1 : 1);
        }
    }


}

==> thesis-1.1-20150615.061726-1-sources.jar: com/whimsy/map/algo/AStar.java <==
package com.whimsy.map.algo;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.PriorityQueue;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.whimsy.map.base.Constant;
import com.whimsy.map.api.Edge;
import com.whimsy.map.api.Graph;
import com.whimsy.map.api.Node;

import edu.princeton.cs.algs4.Bag;


import static com.whimsy.map.base.Utils.*;

/**
 * Created by whimsy on 6/1/15.
 */
public class AStar {
    static final Logger LOG = LoggerFactory.getLogger(AStar.class);

    Graph graph;

    Bag<Edge>[] bags;

    List<Node> nodes;
    List<Edge> edges;

    public AStar(Graph graph) {
        this.graph = graph;
        this.nodes = graph.nodes;
        this.edges = graph.edges;

        bags = new Bag[nodes.size()];

        for (int i = 0; i < nodes.size(); ++i) {
            bags[i] = new Bag<Edge>();
        }

        for (Edge edge : edges) {
            bags[edge.sId].add(edge);
        }
    }



    /**
     * 返回最短路径的路径规划
     * @param s 起点
     * @param t 终点
     * @return  点的数组
     */
    public Node[] findPath(int s, int t) {

        PriorityQueue<HeapNode> pq = new PriorityQueue<HeapNode>();

        int n = bags.length;


        double [] d = new double[n];
        for (int i = 0; i < n; ++i) {
            d[i] = Constant.INF;
        }

        int [] from = new int[n];
        from[s] = -1;

        Edge [] preE = new Edge[n];



        // initialize source node

        d[s] = 0;
        pq.add(new HeapNode(s, d[s] + estimateDist(s, t)));

        boolean [] isClosed = new boolean[n];

        while (!pq.isEmpty()) {
            HeapNode cur = pq.poll();

            if (isClosed[cur.idx]) continue;

            isClosed[cur.idx] = true;
            if (cur.idx == t) {
                break;
            }

            for (Edge edge : bags[cur.idx]) {
                if (d[edge.eId] > d[cur.idx] + edge.len()) {
                    d[edge.eId]  = d[cur.idx] + edge.len();
                    pq.add(new HeapNode(edge.eId, d[edge.eId] + estimateDist(edge.eId, t)));
                    preE[edge.eId] = edge;
                }
            }
        }

        LOG.debug("AStar distance : {}", d[t]);


        // calc path
        ArrayList<Edge> reverseEdge = new ArrayList<Edge>();

        int cur = t;
        if (d[cur] != Constant.INF) {
            while (preE[cur] != null) {
                reverseEdge.add(preE[cur]);
                cur = preE[cur].sId;
            }
        }

        Collections.reverse(reverseEdge);

        ArrayList<Node> retArr = new ArrayList<Node>();

        for (Edge edge : reverseEdge) {
            for (Edge.Figure edgeNode : edge.figures) {

                retArr.add(new Node(edgeNode.lat, edgeNode.lon));
            }
        }

        Node [] res = new Node[retArr.size()];

        retArr.toArray(res);

        return res;
    }

    public double query(int s, int t) {

        PriorityQueue<HeapNode> pq = new PriorityQueue<HeapNode>();

        int n = bags.length;


        double [] d = new double[n];
        for (int i = 0; i < n; ++i) {
            d[i] = Constant.INF;
        }



        // initialize source node

        d[s] = 0;
        pq.add(new HeapNode(s, d[s] + estimateDist(s, t)));

        boolean [] isClosed = new boolean[n];

        while (!pq.isEmpty()) {
            HeapNode cur = pq.poll();

            if (isClosed[cur.idx]) continue;

            isClosed[cur.idx] = true;
            if (cur.idx == t) {
                break;
            }

            for (Edge edge : bags[cur.idx]) {
                if (d[edge.eId] > d[cur.idx] + edge.len()) {
                    d[edge.eId]  = d[cur.idx] + edge.len();
                    pq.add(new HeapNode(edge.eId, d[edge.eId] + estimateDist(edge.eId, t)));
                }
            }
        }

        LOG.debug("AStar distance : {}", d[t]);


        return d[t];

    }

    private double estimateDist(int s, int t) {
        return dist(nodes.get(s).lat, nodes.get(s).lon, nodes.get(t).lat, nodes.get(t).lon);

    }

    class HeapNode implements  Comparable<HeapNode> {

        int idx;
        double value;

        public HeapNode(int idx, double value) {
            this.idx = idx;
            this.value = value;
        }

        @Override
        public int compareTo(HeapNode o) {
            return value == o.value ? 0 : (value < o.value ? -1 : 1);
        }
    }


}
